function [bp,rp,Vmin,Vmax]=calculate_partitions(varargin)
%@CPM\CALCULATE_PARTITIONS
% [bp,rp,Vmin,Vmax]=calculate_partitions(child)
%           child = an instance of a childclass of the parent class cpm.
% [bp,rp,Vmin,Vmax]=calculate_partitions(child,bp,rp,x_min0,x_max0)
%           child = an instance of a childclass of the parent class cpm.
%           bp and rp: input partitions of the same problem as in child but
%                      with other endpoints.
%           x_min0 / x_max0 : old endpoints of the integration interval
%                             (associated to input bp and rp)  
%             --> bp and rp have to be enlarged (or diminished) to reach 
%                 new endpoints (of the problem in child)   
% Stage 1: generates the partition and the associated set of 
% quantities (bp.h, bp.cu, bp.cup, bp.cv, bp.cvp, bp.Vbar, bp.imatch). 
% In addition the reference partition (rp.h = bp.h, rp.cu, rp.cup, rp.cv, rp.cvp, 
% rp.Vbar=bp.Vbar,rp.imatch=rp.imatch) is calculated. 

global cpm_child 
cpm_child = varargin{1};
schr = get(cpm_child,'schrod'); %the schrodinger problem
transf_slp=isa(schr,'transformed_slp');
singularities = get(schr,'singularities'); %this array is empty when there are no singularities
pot = get(schr,'V'); %the potential
x_min = get(schr,'min');
x_max = get(schr,'max');
ref_cpm = get(cpm_child,'ref_cpm');
tol = get(cpm_child,'tol');

inf_max=false;
inf_min=false;
if isinf(x_max)
    inf_max=true;  %endpoint b is infinite
end
if isinf(x_min)
    inf_min=true;  %endpoint a is infinite  
end
if nargin<5
 if inf_min || inf_max 
    bp=[];
    rp=[];
    Vmin=[];
    Vmax=[];
    return; %infinite integration interval: the integration interval must be truncated before the partition can be constructed
 elseif ~isempty(singularities)
    if singularities(1)==x_min && singularities(2)==x_max
       bp=[];
       rp=[];
       Vmin=[];
       Vmax=[];
       return; %singularities: the integration interval must be truncated before the partition can be constructed
    end
 end
end


if nargin==1
%calculate partition:
% this furnishes the vector of stepsizes (bp.h) and the elements of the one
% step propagator (matrices bp.cu, bp.cup, bp.cv, bp.cvp) + bp.Vbar
% and bp.imatch (the interval at the rhs of which is the matching point)
x = x_min;
i=0;
rp.h=[];
trial_h = (x_max-x_min)/50;
if ~isreal(x_max) && imag(x_max) > 0
    error('no success');
end
if isa(cpm_child,'cpm18_16')
    %the reference partition is constructed by halving each interval of the
    %basic partition
 while x < x_max
    i=i+1;
    [h,bp.V0(i),V] = get_step(x,trial_h,x_max-x,tol);
    bp.h(i)=h;
    [bp.cu(i,:),bp.cup(i,:),bp.cv(i,:),bp.cvp(i,:)] = get_coeff(cpm_child,V); 
    rp.h=[rp.h h/2 h/2];
    t=2*i-1;
    [rp.V0(t),V] = calculate_V(x,h/2);
    [rp.cu(t,:),rp.cup(t,:),rp.cv(t,:),rp.cvp(t,:)] = get_coeff(cpm_child,V); 
    t=2*i;
    [rp.V0(t),V] = calculate_V(x+h/2,h/2);
    [rp.cu(t,:),rp.cup(t,:),rp.cv(t,:),rp.cvp(t,:)] = get_coeff(cpm_child,V); 
    x = x + h;
    trial_h =h;
 end   
else
     %reference partition = basic partition but with a higher order method
 while x < x_max
    i=i+1;
    [h(i),bp.V0(i),V] = get_step(x,trial_h,x_max-x,tol);
    [cu(:,i),cup(:,i),cv(:,i),cvp(:,i)] = get_coeff(cpm_child,V); 
    [rcu(:,i),rcup(:,i),rcv(:,i),rcvp(:,i)] = get_coeff(ref_cpm,V); 
    x = x + h(i);
    trial_h = h(i);
 end
 bp.h=h;
 bp.cu=cu';  %omdat kolommen toevoegen sneller gaat dan rijen toevoegen
 bp.cup=cup';
 bp.cv=cv';
 bp.cvp=cvp';
 rp.cu=rcu';
 rp.cup=rcup';
 rp.cv=rcv';
 rp.cvp=rcvp';
end

else %nargin==5
    % part of the partition has already been calculated, it only has to be 
    % enlarged or diminished now.  
   if isa(cpm_child,'cpm18_16')
    bp = varargin{2};
    rp = varargin{3};
    x_min0 = varargin{4};
    x_max0 = varargin{5};
    if x_min0 < x_min
        %make the integration interval + mesh smaller
        x=x_min0;
        i=0;
        while x < x_min
            i=i+1;
            x = x + bp.h(i);
        end
        i=i+1;
        bp.h = bp.h(i:end);
        bp.V0 = bp.V0(i:end);
        bp.cu = bp.cu(i:end,:);
        bp.cv = bp.cv(i:end,:);
        bp.cup = bp.cup(i:end,:);
        bp.cvp = bp.cvp(i:end,:);
        t=2*i-1;
        rp.cu = rp.cu(t:end,:);
        rp.cv = rp.cv(t:end,:);
        rp.cup = rp.cup(t:end,:);
        rp.cvp = rp.cvp(t:end,:);
        x_min0 = x;
    end
    if x_max0 > x_max
        %make the integration interval + mesh smaller
        x=x_max0;
        i=length(bp.h)+1;
        while x > x_max
            i=i-1;
            x = x - bp.h(i);
        end
        i=i-1;
        t=2*i;
        bp.h = bp.h(1:i);
        bp.V0 = bp.V0(1:i);
        bp.cu = bp.cu(1:i,:);
        bp.cv = bp.cv(1:i,:);
        bp.cup = bp.cup(1:i,:);
        bp.cvp = bp.cvp(1:i,:);
        rp.cu = rp.cu(1:t,:);
        rp.cv = rp.cv(1:t,:);
        rp.cup = rp.cup(1:t,:);
        rp.cvp = rp.cvp(1:t,:);
        x_max0 = x;
    end
    if x_min0 > x_min
      x = x_min;
      trial_h = bp.h(1);  
      %enlarge the mesh
      i=0;
      rp0.h=[];
      while x < x_min0
        i=i+1;
        [h,bp0.V0(i),V] = get_step(x,trial_h,x_min0-x,tol);
        bp0.h(i)=h;
        [bp0.cu(i,:),bp0.cup(i,:),bp0.cv(i,:),bp0.cvp(i,:)] = get_coeff(cpm_child,V); 
        rp0.h=[rp0.h h/2 h/2];
        t=2*i-1;
        [rp0.V0(t),V] = calculate_V(x,h/2);
        [rp0.cu(t,:),rp0.cup(t,:),rp0.cv(t,:),rp0.cvp(t,:)] = get_coeff(cpm_child,V); 
        t=2*i;
        [rp0.V0(t),V] = calculate_V(x+h/2,h/2);
        [rp0.cu(t,:),rp0.cup(t,:),rp0.cv(t,:),rp0.cvp(t,:)] = get_coeff(cpm_child,V); 
        x = x + bp0.h(i);
        trial_h = bp0.h(i);
      end 
      bp.h = [bp0.h bp.h];
      bp.V0 = [bp0.V0 bp.V0];
      bp.cu = [bp0.cu;bp.cu];
      bp.cup = [bp0.cup;bp.cup];
      bp.cv = [bp0.cv;bp.cv];
      bp.cvp = [bp0.cvp;bp.cvp];
      rp.cu = [rp0.cu;rp.cu];
      rp.cup = [rp0.cup;rp.cup];
      rp.cv = [rp0.cv;rp.cv];
      rp.cvp = [rp0.cvp;rp.cvp];
    end
    if x_max0 < x_max
        %enlarge the mesh
      x = x_max0;
      i=length(bp.h);
      trial_h = bp.h(end);  
      while x < x_max
        i=i+1;
        [bp.h(i),bp.V0(i),V] = get_step(x,trial_h,x_max-x,tol);
        [bp.cu(i,:),bp.cup(i,:),bp.cv(i,:),bp.cvp(i,:)] = get_coeff(cpm_child,V); 
        rp.h=[rp.h h/2 h/2];
        t=2*i-1;
        [rp.V0(t),V] = calculate_V(x,h/2);
        [rp.cu(t,:),rp.cup(t,:),rp.cv(t,:),rp.cvp(t,:)] = get_coeff(cpm_child,V); 
        t=2*i;
        [rp.V0(t),V] = calculate_V(x+h/2,h/2);
        [rp.cu(t,:),rp.cup(t,:),rp.cv(t,:),rp.cvp(t,:)] = get_coeff(cpm_child,V); 
        x = x + bp.h(i);
        trial_h = bp.h(i);
      end
    end
   else
    bp = varargin{2};
    rp = varargin{3};
    x_min0 = varargin{4};
    x_max0 = varargin{5};
    if x_min0 < x_min
        %make the integration interval + mesh smaller
        x=x_min0;
        i=0;
        while x < x_min
            i=i+1;
            x = x + bp.h(i);
        end
        i=i+1;
        bp.h = bp.h(i:end);
        bp.V0 = bp.V0(i:end);
        bp.cu = bp.cu(i:end,:);
        bp.cv = bp.cv(i:end,:);
        bp.cup = bp.cup(i:end,:);
        bp.cvp = bp.cvp(i:end,:);
        rp.cu = rp.cu(i:end,:);
        rp.cv = rp.cv(i:end,:);
        rp.cup = rp.cup(i:end,:);
        rp.cvp = rp.cvp(i:end,:);
        x_min0 = x;
    end
    if x_max0 > x_max
        %make the integration interval + mesh smaller
        x=x_max0;
        i=length(bp.h)+1;
        while x > x_max
            i=i-1;
            x = x - bp.h(i);
        end
        i=i-1;
        bp.h = bp.h(1:i);
        bp.V0 = bp.V0(1:i);
        bp.cu = bp.cu(1:i,:);
        bp.cv = bp.cv(1:i,:);
        bp.cup = bp.cup(1:i,:);
        bp.cvp = bp.cvp(1:i,:);
        rp.cu = rp.cu(1:i,:);
        rp.cv = rp.cv(1:i,:);
        rp.cup = rp.cup(1:i,:);
        rp.cvp = rp.cvp(1:i,:);
        x_max0 = x;
    end
    if x_min0 > x_min
      x = x_min;
      trial_h = bp.h(1);  
      %enlarge the mesh
      i=0;
      while x < x_min0
        i=i+1;
        [bp0.h(i),bp0.V0(i),V] = get_step(x,trial_h,x_min0-x,tol);
        [bp0.cu(i,:),bp0.cup(i,:),bp0.cv(i,:),bp0.cvp(i,:)] = get_coeff(cpm_child,V); 
        [rp0.cu(i,:),rp0.cup(i,:),rp0.cv(i,:),rp0.cvp(i,:)] = get_coeff(ref_cpm,V); 
        x = x + bp0.h(i);
        trial_h = bp0.h(i);
      end 
      bp.h = [bp0.h bp.h];
      bp.V0 = [bp0.V0 bp.V0];
      bp.cu = [bp0.cu;bp.cu];
      bp.cup = [bp0.cup;bp.cup];
      bp.cv = [bp0.cv;bp.cv];
      bp.cvp = [bp0.cvp;bp.cvp];
      rp.cu = [rp0.cu;rp.cu];
      rp.cup = [rp0.cup;rp.cup];
      rp.cv = [rp0.cv;rp.cv];
      rp.cvp = [rp0.cvp;rp.cvp];
    end
    if x_max0 < x_max
        %enlarge the mesh
      x = x_max0;
      i=length(bp.h);
      trial_h = bp.h(end);  
      while x < x_max
        i=i+1;
        [bp.h(i),bp.V0(i),V] = get_step(x,trial_h,x_max-x,tol);
        [bp.cu(i,:),bp.cup(i,:),bp.cv(i,:),bp.cvp(i,:)] = get_coeff(cpm_child,V); 
        [rp.cu(i,:),rp.cup(i,:),rp.cv(i,:),rp.cvp(i,:)] = get_coeff(ref_cpm,V); 
        x = x + bp.h(i);
        trial_h = bp.h(i);
      end
    end
   end
end

%rp: reference partition (used to form an estimation of the error in the
%calculated eigenvalues): has equal stepsizes and meshpoints as the basic 
%partition (bp) but a higher order method is used to advance the solution (tenzij CPM{18,16}).
[Vmin,bp.imatch] = min(bp.V0);
Vmax = max(bp.V0);
if isa(cpm_child,'cpm18_16')
  rp.imatch = bp.imatch*2-1;
else
  rp.h = bp.h;  
  rp.V0 = bp.V0;  
  rp.imatch = bp.imatch;
end


%if symmetric problem, it is nice to have the matching point in the middle:
symm = true;
if ~isempty(singularities) &&  (abs(x_min-singularities(1))<1e-5 || abs(x_max-singularities(2))<1e-5)
     if ~isinf(singularities(1)) 
         x_min=singularities(1);
     end
     if ~isinf(singularities(2)) 
         x_max=singularities(2);
     end
end
if transf_slp
 mid = (x_max+x_min)/2;
 for i=0:(x_max-x_min)/50:mid
  r1=x2r(schr,mid+i);
  r2=x2r(schr,mid-i);
  if pot(r1) ~= pot(r2)
        symm =false;
        break;
  end
 end   
else
 mid = (x_max+x_min)/2;
 for i=0:(x_max-x_min)/200:mid
  if pot(mid+i) ~= pot(mid-i)
        symm =false;
        break;
  end
 end
end
if symm
      sum = x_min;
      for i=1:length(bp.h)
          sum = sum + bp.h(i); 
          if sum > mid
              break;
          end
      end
      bp.imatch = i;
      rp.imatch = bp.imatch;
      if isa(cpm_child,'cpm18_16')
          rp.imatch = bp.imatch*2-1;
      end
end
    
