function [varargout] = get_eigenfunction(varargin)
%@PPM\GET_EIGENFUNCTION
% V = get_eigenfunction(ppm_child,e)
% [V,status,part] = get_eigenfunction(ppm_child,e,n)
% [V,status,part] = get_eigenfunction(ppm_child,e,part) 
    %part: (in input) partition which may have to be enlarged.
    %part: (in output) partition on which the eigenfunction is evaluated
% [V,status,part] = get_eigenfunction(ppm_child,e,part,false)
   % the integration interval may not be truncated
% [V,status,part] = get_eigenfunction(ppm_child,e,a,b,n)
% returns (part of) the eigenfunction corresponding with eigenvalue e.
% 
% To obtain a good approximation for the eigenfunction
% the eigenvalue e must be sufficiently accurate. 
% (choose a low value for tol)
%
% If n is omitted, then the eigenfunction is only evaluated in 
% the meshpoints of the partition (bp).
% If n > 0, then the interval [a,b] is taken and further
% partitioned in n intervals. The eigenfunction is then evaluated only in
% the endpoints of these n intervals.
% When the input-arguments a and b are omitted, then the whole interval 
% integration interval [x_min,x_max] is considered.
% When a == b, the value of the eigenfunction in a is returned.
%
%    V.x:                    - a vector with the values of the points
%                             where the eigenfunction is evaluated. 
%    V.y:                    - a vector which contains the values at the 
%                             points collected in V.x of the eigenfunction 
%                             associated to the eigenvalue e.
%    V.yprime:               - a vector which contains the values at the 
%                             points collected in V.x of the first 
%                             derivative of the eigenfunction associated 
%                             to the eigenvalue e.
%

global ppm_child schrod_object
ppm_child = varargin{1};
schrod_object = get(ppm_child,'schrod');
pot=get(schrod_object,'V');
truncated = get(ppm_child,'truncated');
distorted=isa(schrod_object,'distorted_coulomb');
e = varargin{2};
status = 0;

switch nargin
    case 2
        [V.x,V.y,V.yprime,status] = calculate_eigenfunction(e,get(ppm_child,'bp'));  
    case {3,4,5}
        if nargin == 4
            n = varargin{3};
        else
            n = varargin{nargin};
        end
        if nargin == 3 || nargin ==4
              if isa(n,'double')
               if truncated && nargin == 3
                   bp=get(ppm_child,'bp');
                   x_min=get(schrod_object,'min');
                   x_max=get(schrod_object,'max');
                   [trunca,truncb]=get_truncation_points(bp,e,distorted);
                   x_a=x_min+sum(bp.h(1:trunca-1));
                   x_b=x_min+sum(bp.h(1:truncb));
                   if abs(abs(x_a)-x_b) < (x_b/5) && x_a*x_b<0 && pot(x_a)==pot(-x_a)
                       x_b=min(max(abs(x_a),x_b),x_max);
                       x_a=max(-x_b,x_min);
                   end
                   if distorted
                     part=get_ppm(ppm_child,'partition',e,x_min,x_b,n);
                     [V.x,V.y,V.yprime,status] = calculate_eigenfunction(e,part,0,x_b);  
                   else
                     part=get_ppm(ppm_child,'partition',e,x_a,x_b,n);
                     [V.x,V.y,V.yprime,status] = calculate_eigenfunction(e,part,x_a,x_b);
                   end
               else
                 part=get_ppm(ppm_child,'partition',e,n);
                 [V.x,V.y,V.yprime,status] = calculate_eigenfunction(e,part);
               end
              else
                  if distorted
                    part=get_ppm(ppm_child,'partition',e,n);
                  else
                    part=n;
                  end
                  [V.x,V.y,V.yprime,status] = calculate_eigenfunction(e,part);
              end
        elseif nargin ==5
            if isa(schrod_object,'transformed_slp')
                 x = r2x(schrod_object,[varargin{3},varargin{4}]);
                 part=get_ppm(ppm_child,'partition',e,x(1),x(2),n);
                 [V.x,V.y,V.yprime,status] = calculate_eigenfunction(e,part,x(1),x(2));
            else
                 if varargin{3} == varargin{4}
                     error('invalid interval');
                 end
                 if distorted
                     x_min=get(schrod_object,'min');
                     part=get_ppm(ppm_child,'partition',e,max(varargin{3},x_min),varargin{4},n);
                     [V.x,V.y,V.yprime,status] = calculate_eigenfunction(e,part,varargin{3},varargin{4});
                 else
                     part=get_ppm(ppm_child,'partition',e,varargin{3},varargin{4},n);
                     [V.x,V.y,V.yprime,status] = calculate_eigenfunction(e,part,varargin{3},varargin{4});
                 end
            end
        end
   otherwise
       error('ppm\get_eigenfunction: 2 or 3 or 5 inputarguments expected')
end
if isa(schrod_object,'transformed_slp')
 [V.x,V.y,V.yprime] = transform_eigenfunction(schrod_object,V.x,V.y,V.yprime);
end
varargout{1}=V;
if nargout >= 2
    varargout{2}= status;
    if nargout ==3
        varargout{3} = part;
    end
end
         

function [trunca,truncb]=get_truncation_points(bp,E,distorted)
%determination of truncation points
trunca=1;
truncb=length(bp.h);
inita=bp.imatch;
initb=bp.imatch;
  sums=0;
  V0=bp.V0;
  h=bp.h;
  for i=initb:length(h)-1 
    v0i=V0(i);  
    if v0i>=E && V0(i+1) > v0i
      sums=sums+sqrt(v0i-E)*h(i);
    else
      sums=0;  
    end
    if sums > 18
        truncb=i;
        for j=length(h):-1:i+1
            if V0(j)>=v0i && V0(j-1)<v0i
                truncb=j;
            end
        end
        break;
    end
  end
if ~distorted
  sums=0;
  for i=inita:-1:2
    v0i=V0(i);  
    if v0i>=E && V0(i-1) > v0i
      sums=sums+sqrt(v0i-E)*h(i);
    else
      sums=0;  
    end
    if sums > 18
        trunca=i;
        for j=1:i-1
            if V0(j)>=v0i && V0(j+1)<v0i
                trunca=j;
            end
        end
        break;
    end
  end
end

