function varargout = get_eigenvalues(varargin)
%@PPM\GET_EIGENVALUES
% E = get_eigenvalues(child,min,max)
% E = get_eigenvalues(child,min,max,indices)
% or
% [E,child] = get_eigenvalues(child,min,max)
% [E,child] = get_eigenvalues(child,min,max,indices)
% returns the eigenvalues (E) of the problem in child.
% The child in the output is different from the child in the output
% only when a or b =inf
%
% indices       - boolean:
%                  indices = true: 
%                       min = lower index,
%                       max = upper index 
%                    for the eigenvalues to be searched for
%                 otherwise (indices = false or indices is omitted): 
%                       min = lower limit,
%                       max = upper limit
%                    of the interval to be scanned for eigenvalues
%
%    E.success:               =true if the CP=method ran successfully
%                             =false: the CP-method wasn't able to obtain the data
%                              due to an error. E.msg contains the error message.
%    E.eigenvalues:           a vector with the eigenvalues in the ascending
%                             order. 
%    E.errors:                a vector with the errors in the eigenvalues 
%                             calculated on the partition.
%    E.indices:               a vector which contains the index n of each eigenvalue.
%    E.status:                a vector of status-flags, E.status(i) furnishes
%                             information on the quality of the eigenvalue i:
%            E.status(i) = 0       - correct calculation
%            E.status(i) = 1       - at least one of the eigenvalues 
%                                    E.basic_eigenvalues(i) and 
%                                    E.reference_eigenvalues(i) or of their indices 
%                                    is inaccurate for reasons which may basically
%                                    suggest that the double precision is not sufficient 
%                                    for that case.
%            E.status(i) = 2        - the indices of E.basic_eigenvalues(i) and 
%                                     E.reference_eigenvalues(i) are different.
%            E.status(i) = 3        - the index of E.basic_eigenvalues(i) is uncertain.
%            E.status(i) = 4        - = E.status(i) = 2 + E.status(i) = 3
%            E.status(i) > 0 may suggest that the input tol should be decreased.  

global ppm_child schrod_object 

switch nargin
    case {3,4}
        ppm_child = varargin{1};
        schrod_object = get(ppm_child,'schrod');
        if nargin == 3
          indices = false;
        else
          indices = varargin{4};
        end
    otherwise
        error('get_eigenvalues: 3 or 4 inputarguments expected')
end

classif=true;
try
   class=get(schrod_object,'classification');
catch
    classif=false;
end
if isempty(class)
    classif=false;
end

if indices
    indmin=varargin{2};
    indmax=varargin{3};
    if indmin < 0 || indmax < 0
        error('Error!!! : The indices must be positive.');
    elseif indmin > indmax 
        error('Error!!! : The lower index must be smaller than or equal to the upper index');
    elseif floor(indmin) ~= indmin || floor(indmax) ~= indmax
        error('Error!!! : The indices must be integer values');
    end
else
    if varargin{2} >= varargin{3} 
          error('Error!!! : The lower energy-value must be smaller than the upper energy-value');
    end
end

if classif
    varargin{3}=check_evrange(class,varargin{2},varargin{3},indices);
end

singularities=get(schrod_object,'singularities');
if isinf(get(schrod_object,'min')) || isinf(get(schrod_object,'max')) || ~isempty(singularities)
    %truncate the problem
    if isa(schrod_object,'transformed_slp')
       error('This Sturm-Liouville problem defined on an infinite integration interval can not be solved in MATSLISE.')
    else
       ppm_child=get_ppm(ppm_child,'regular_problem',varargin{3},indices);
    end
    if nargout >1
            varargout{2}=ppm_child;
    end
end
% get_intervals returns the energy range [emin,emax] and the index range 
% [nmin,nmax] of the eigenvalues to be obtained. 
[emin,emax,nmin,nmax,err] = get_intervals(varargin{2},varargin{3},indices);
if nmax-nmin<0
 if classif && ( (class.category==2 || class.category==10 ) && emax == class.cutoff)
    nmax=class.lastev-1;
 end
end   

if err.fail
   E.eigenvalues=[];
   E.indices=[];
   E.errors=[];
   E.status=[]; 
   E.success = false;
   E.msg = err.msg ;
elseif nmax - nmin < 0 
   E.eigenvalues=[];
   E.indices=[];
   E.errors=[];
   E.status=[]; 
   E.success = false;
   E.msg ='no eigenvalue is found in the given interval' ;
else
   % The effective search of the eigenvalues with index n from nmin to nmax is
   % now done.
   singularities=get(schrod_object,'singularities');
   if isempty(singularities)
    E=get_ppm(ppm_child,'eigenvalues',emin,emax,nmin,nmax);
   elseif isa(schrod_object,'transformed_slp') && isempty(rG)
    error('This singular Sturm-Liouville problem can not be solved in MATSLISE.')
   else
     E=get_ppm(ppm_child,'eigenvalues_singular',emin,emax,nmin,nmax);
   end
   E.success = ~err.fail;
end
varargout{1}=E;
if nargout >1
    varargout{2}=ppm_child;
end



%---------------------------------------------
function [emax]=check_evrange(class,emin,emax,indices)
if indices
  if class.category==2 || class.category==10
   lastev=class.lastev;
   if lastev>=0 && emax>=lastev
        if emin < lastev
            emax=lastev-1;
            if lastev == 0
               warndlg('There appear to be no eigenvalues below the start of the continuous spectrum.');
            else
               warndlg(['There are only ' num2str(lastev) ' eigenvalues in the discrete spectrum. Only the eigenvalues with index < ' num2str(lastev) ' are calculated.']);
            end
        else
            if lastev == 0
               error('Error!!! : There appear to be no eigenvalues below the start of the continuous spectrum.');
            else
               error(['Error!!! : There are only ' num2str(lastev) ' eigenvalues in the discrete spectrum.']);
            end
        end
   end
  end
else
    cutoff=class.cutoff;
    if emax > cutoff
        if emin < cutoff
            emax = cutoff;
        else
            error('Error!!! : The requested energy range is situated in the continuous spectrum.');
        end
    end
    if class.category==2 || class.category==10
        lastev=class.lastev;
        if lastev == 0
            error('Error!!! : There appear to be no eigenvalues below the start of the continuous spectrum.');
        end
    end
end